BITFINEX = 'bitfinex'
OKEX = 'okex'
KRAKEN = 'kraken'

EXCHANGES = [
    BITFINEX,
    OKEX,
    KRAKEN,
]

BUY = 'buy'
SELL = 'sell'
ACTIONS = [
    BUY,
    SELL
]

LIMIT = 'limit'
MARKET = 'market'
ORDER_TYPES = [
    LIMIT,
    MARKET
]



from decimal import Decimal

from tests import BaseXchangeTestCase
from xchange.constants import currencies
from xchange.models.utils import (
    object_of_class, sorted_list, restricted_to_values,
    normalized_symbol, normalized_symbol_pair,
    contracts_to_crypto, crypto_to_contracts
)

class ValidatorsTestCase(BaseXchangeTestCase):

    def test_object_of_class(self):
        func = object_of_class('Decimal')
        func(Decimal('10.2'))
        with self.assertRaises(ValueError):
            func('Hello World!')

    def test_sorted_list_asc(self):
        func = sorted_list(lambda x: x['value'], 'asc')
        l = [
            {'value': 10, 'result': True},
            {'value': 100, 'result': False},
            {'value': 30, 'result': True},
        ]
        result = func(l)
        self.assertEqual(
            result,
            [
                {'result': True, 'value': 10},
                {'result': True, 'value': 30},
                {'result': False, 'value': 100}
            ]
        )

    def test_sorted_list_desc(self):
        func = sorted_list(lambda x: x['value'], 'desc')
        l = [
            {'value': 10, 'result': True},
            {'value': 100, 'result': False},
            {'value': 30, 'result': True},
        ]
        result = func(l)
        self.assertEqual(
            result,
            [
                {'result': False, 'value': 100},
                {'result': True, 'value': 30},
                {'result': True, 'value': 10}
            ]
        )

    def test_restricted_to_values(self):
        func = restricted_to_values(['Hello', 'World'])
        func('Hello')
        func('World')
        with self.assertRaises(ValueError):
            func('foobar')


class NormalizersTestCase(BaseXchangeTestCase):

    def test_normalized_symbol(self):
        self.assertEqual(normalized_symbol(currencies.USD), currencies.USD)
        self.assertEqual(normalized_symbol('zusd'), currencies.USD)
        with self.assertRaises(ValueError):
            normalized_symbol('foobar')

    def test_normalized_symbol_pair(self):
        self.assertEqual(normalized_symbol_pair(currencies.BTC_USD), currencies.BTC_USD)
        self.assertEqual(normalized_symbol_pair('xbtusd'), currencies.BTC_USD)
        with self.assertRaises(ValueError):
            normalized_symbol_pair('foobar')


class ConvertersTestCase(BaseXchangeTestCase):

    def test_contracts_to_crypto(self):
        amount_in_crypto = contracts_to_crypto(
            amount_in_contracts=3,
            crypto_last_price=8000,
            unit_amount=100
        )
        # amount_in_crypto = amount_in_contracts * (unit_amount / crypto_last_price)
        # 0.0375 = 3 * (100 / 8000)
        self.assertEqual(amount_in_crypto,  Decimal('0.0375'))

    def test_crypto_to_contracts(self):
        amount_in_contracts = crypto_to_contracts(
            amount_in_crypto=Decimal('0.0375'),
            crypto_last_price=8000,
            unit_amount=100
        )
        # amount_in_contracts = (amount_in_crypto * crypto_last_price) / unit_amount
        # 3 = (0.0375 * 8000) / 100
        self.assertEqual(amount_in_contracts,  3)



from decimal import Decimal

from xchange.constants import currencies


def as_decimal(value):
    return Decimal(str(value))


def object_of_class(class_name):
    def func(obj):
        if obj.__class__.__name__ != class_name:
            raise ValueError('"{}" is not a valid class'.format(obj))
        return obj
    return func


def sorted_list(key, sorting_type):
    def func(original_list):
        reverse = False if sorting_type == 'asc' else True
        return sorted(original_list, key=key, reverse=reverse)
    return func


def restricted_to_values(values_list):
    values_list = [elem.lower() for elem in values_list]
    def func(original_value):
        original_value = original_value.lower()
        if original_value not in values_list:
            raise ValueError('"{}" is not a valid value'.format(original_value))
        return original_value
    return func


def normalized_symbol(original_symbol):
    mapping = currencies.SYMBOL_VARIANTS
    original_symbol = original_symbol.lower()
    if original_symbol in mapping.keys():
        # no need to normalize, symbol is already valid
        return original_symbol
    for symbol, variants in mapping.items():
        if original_symbol in variants:
            return symbol
    raise ValueError('Could not normalize {} symbol'.format(original_symbol))


def normalized_symbol_pair(original_pair):
    mapping = currencies.SYMBOL_PAIR_VARIANTS
    original_pair = original_pair.lower()
    if original_pair in mapping.keys():
        # no need to normalize, symbol pair is already valid
        return original_pair
    for pair, variants in mapping.items():
        if original_pair in variants:
            return pair
    raise ValueError('Could not normalize {} symbol pair'.format(original_pair))


def contracts_to_crypto(amount_in_contracts, crypto_last_price, unit_amount):
    """
    Transforms the amount of contracts to amount of given cryptos based
    on the `unit_amount` and last price of the coin.

    `unit_amount` is the price in USD of each contract.

    amount_in_crypto = amount_in_contracts * (unit_amount / crypto_last_price)
    """
    # convert all given values to Decimal
    amount_in_contracts, crypto_last_price, unit_amount = list(
        map(Decimal, [amount_in_contracts, crypto_last_price, unit_amount]))
    return amount_in_contracts * (unit_amount / crypto_last_price)


def crypto_to_contracts(amount_in_crypto, crypto_last_price, unit_amount):
    """
    Transforms the amount of given crypto to amount of contracts based
    on the `unit_amount` and last price of the coin.
    It rounds the amount of contracts to the nearest lower amount,
    because contracts can not be operated partially.
    ie: If contracts amount ends up being Decimal(1.6873),
        it returns Decimal(1).

    `unit_amount` is the price in USD of each contract.

    amount_in_contracts = (amount_in_crypto * crypto_last_price) / unit_amount
    """
    # convert all given values to Decimal
    amount_in_crypto, crypto_last_price, unit_amount = list(
        map(Decimal, [amount_in_crypto, crypto_last_price, unit_amount]))
    amount_in_contracts = (amount_in_crypto * crypto_last_price) / unit_amount
    return Decimal(int(amount_in_contracts))



import importlib

from xchange.clients import BitfinexClient
from xchange.constants import exchanges


class ExchangeClientFactory:

    @classmethod
    def get_client(self, exchange_name):
        if not exchange_name in exchanges.EXCHANGES:
            raise NotImplementedError(
                'Client for exchange "{}" is not implemented'.format(exchange_name))

        namespace = 'xchange.clients.{}'.format(exchange_name)
        clients_module = importlib.import_module(namespace)
        ClientClass = getattr(
            clients_module,
            '{}Client'.format(exchange_name.title())
        )
        return ClientClass



from decimal import Decimal

from tests import BaseXchangeTestCase
from xchange.models.base import (
    Ticker, OrderBook
)

class TickerTestCase(BaseXchangeTestCase):

    def test_ticker(self):
        ticker = Ticker({
            'ask': '9314.65',
            'bid': '100.51',
            'high': '9480',
            'last': '9312.34',
            'low': '8800',
            'volume': '16185076',
        })
        self.assertEqual(ticker.ask, Decimal('9314.65'))
        self.assertEqual(ticker.bid, Decimal('100.51'))
        self.assertEqual(ticker.high, Decimal('9480'))
        self.assertEqual(ticker.last, Decimal('9312.34'))
        self.assertEqual(ticker.low, Decimal('8800'))
        self.assertEqual(ticker.volume, Decimal('16185076'))

        with self.assertRaisesRegexp(AttributeError,
                                     'Object Ticker has not attribute "foobar"'):
            ticker.foobar

    def test_ticker_invalid_data(self):
        with self.assertRaisesRegexp(ValueError,
                                     'Unknown field "foo" for class Ticker'):
            Ticker({
                'foo': 'bar'
            })


class OrderBookTestCase(BaseXchangeTestCase):

    def test_order_book(self):
        order_book = OrderBook({
            "asks": [
                # (price_in_btc, amount_in_btc),
                (Decimal('4630.12300'), Decimal('0.014')),
                (Decimal('4620.23450'), Decimal('0.456')),
            ],
            "bids": [
                # (price_in_btc, amount_in_btc),
                (Decimal('4610.54856'), Decimal('0.078')),
                (Decimal('4600.78952'), Decimal('0.125')),
            ]
        })
        self.assertEqual(type(order_book.asks), list)
        self.assertEqual(type(order_book.bids), list)

        with self.assertRaisesRegexp(AttributeError,
                                     'Object OrderBook has not attribute "foobar"'):
            order_book.foobar

    def test_order_book_invalid_data(self):
        with self.assertRaisesRegexp(ValueError,
                                     'Unknown field "foo" for class OrderBook'):
            OrderBook({
                'foo': 'bar'
            })



import logging
import hashlib
from decimal import Decimal

from cached_property import cached_property_with_ttl

from xchange import exceptions
from xchange.constants import currencies, exchanges
from xchange.clients.base import BaseExchangeClient
from xchange.models.base import crypto_to_contracts
from xchange.validators import is_restricted_to_values, is_instance, passes_test
from xchange.models.okex import (
    OkexTicker, OkexOrderBook, OkexAccountBalance, OkexOrder, OkexPosition)


class OkexClient(BaseExchangeClient):
    BASE_API_URL = 'https://www.okex.com/api'
    ERROR_CLASS = exceptions.OkexException
    SYMBOLS_MAPPING = {
        currencies.BTC_USD: 'btc_usd',
        currencies.ETH_USD: 'eth_usd',
        currencies.ETC_USD: 'etc_usd',
        currencies.LTC_USD: 'ltc_usd',
        currencies.BCH_USD: 'bch_usd',
        currencies.XRP_USD: 'xrp_usd',
        currencies.EOS_USD: 'eos_usd',
        currencies.BTG_USD: 'btg_usd',
    }
    ORDER_STATUS = {
        'unfilled': 1,
        'filled': 2
    }
    ACTION = {
        'open_long': 1,
        'open_short': 2,
        'close_long': 3,
        'close_short': 4,
    }
    CONTRACT_UNIT_AMOUNTS = {
        'btc_usd': 100,
        'eth_usd': 10,
        'etc_usd': 10,
        'ltc_usd': 10,
        'bch_usd': 10,
        'xrp_usd': 10,
        'eos_usd': 10,
        'btg_usd': 10,
    }

    def _sign_params(self, params):
        sign = ''
        for key in sorted(params.keys()):
            sign += key + '=' + str(params[key]) + '&'
        data = sign + 'secret_key=' + self.api_secret
        return hashlib.md5(data.encode("utf8")).hexdigest().upper()

    # transformation functions
    def _transform_account_balance(self, json_response):
        """
        Original JSON response:
        {'info': {
              'btc': {
                   'account_rights': 0.01628945,
                   'keep_deposit': 0,
                   'profit_real': -0.00031059,
                   'profit_unreal': 0,
                   'risk_rate': 10000},
              'ltc': {
                   'account_rights': 0,
                   'keep_deposit': 0,
                   'profit_real': 0,
                   'profit_unreal': 0,
                   'risk_rate': 10000}},
              'result': True}
        """
        return [
            {'currency': key, 'amount': value['account_rights']}
            for key, value in json_response['info'].items()
        ]

    def _transform_open_orders(self, json_response):
        """
        Original JSON response:

        # without open orders
        {'orders': [], 'result': True}

        # with open orders
        {'orders': [{
           'amount': 1,
           'contract_name': 'BTC1229',
           'create_date': 1506188792000,
           'deal_amount': 0,
           'fee': 0,
           'lever_rate': 20,
           'order_id': 10602289748,
           'price': 3000,
           'price_avg': 0,
           'status': 0,
           'symbol': 'btc_usd',
           'type': 1,
           'unit_amount': 100}],
         'result': True}
        """
        return json_response['orders']

    def _transform_open_positions(self, json_response):
        """
        Original JSON response:
        {'force_liqu_price': '0.00',
         'holding': [{'buy_amount': 0,
                    'buy_available': 0,
                    'buy_price_avg': 3620.31999724,
                    'buy_price_cost': 3620.31999724,
                    'buy_profit_real': 0,
                    'contract_id': 20171229012,
                    'contract_type': 'quarter',
                    'create_date': 1505609260000,
                    'lever_rate': 20,
                    'sell_amount': 0,
                    'sell_available': 0,
                    'sell_price_avg': 3930.07,
                    'sell_price_cost': 3930.07,
                    'sell_profit_real': 0,
                    'symbol': 'btc_usd'}],
         'result': True}
        """
        positions_data = json_response['holding']
        if not positions_data:
            return []
        positions = []
        if positions_data[0]['buy_amount']:
            positions.append({
                'id': None,
                'action': 'buy',
                'amount': positions_data[0]['buy_amount'],
                'price': positions_data[0]['buy_price_cost'],
                'symbol_pair': positions_data[0]['symbol'],
                'profit_loss': positions_data[0]['buy_profit_real'],
            })
        if positions_data[0]['sell_amount']:
            positions.append({
                'id': None,
                'action': 'sell',
                'amount': positions_data[0]['sell_amount'],
                'price': positions_data[0]['sell_price_cost'],
                'symbol_pair': positions_data[0]['symbol'],
                'profit_loss': positions_data[0]['sell_profit_real'],
            })
        return positions

    # public endpoints

    def get_ticker(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        return self._get('/v1/future_ticker.do?symbol={}&contract_type=quarter'
                         ''.format(symbol_pair), model_class=OkexTicker)

    def get_order_book(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        OkexOrderBook.TICKER = getattr(self, '{}_ticker'.format(symbol_pair))
        OkexOrderBook.SYMBOL = symbol_pair
        OkexOrderBook.CONTRACT_UNIT_AMOUNTS = self.CONTRACT_UNIT_AMOUNTS
        return self._get('/v1/future_depth.do?size=100&symbol={}&contract_type=quarter'
                         ''.format(symbol_pair), model_class=OkexOrderBook)

    # authenticated endpoints

    def get_account_balance(self, symbol=None):
        is_restricted_to_values(symbol, currencies.SYMBOLS + [None])

        path = '/v1/future_userinfo.do'
        params = {}
        params['api_key'] = self.api_key
        params['sign'] = self._sign_params(params)
        data = self._post(path, params=params,
                          transformation=self._transform_account_balance,
                          model_class=OkexAccountBalance)
        if symbol is None:
            return data
        for symbol_balance in data:
            if symbol_balance.symbol == symbol:
                return symbol_balance
        raise self.ERROR_CLASS('Symbol "{}" was not found in the account balance'.format(symbol))

    def get_open_orders(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        path = '/v1/future_order_info.do'
        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        params = {
            'symbol': symbol_pair,
            'contract_type': 'quarter',
            'status': self.ORDER_STATUS['unfilled'],
            'order_id': -1,  # all orders with given "status"
            'current_page': 1,
            'page_length': 50,
        }
        params['api_key'] = self.api_key
        params['sign'] = self._sign_params(params)
        data = self._post(path, params=params,
                          transformation=self._transform_open_orders,
                          model_class=OkexOrder)
        return [order for order in data
                if order['symbol_pair'] == symbol_pair]

    def open_order(self, action, amount, symbol_pair, price, order_type,
                   amount_in_contracts=False, closing=False):
        """
        Creates a new Order.

        :action:
            exchanges.ACTIONS choice
        :amount:
            Decimal, float, integer or string representing number value.
            If `amount_in_contracts == True`, `amount` needs to be an
            integer number greater or equal to 1.
        :symbol_pair:
            currencies.SYMBOL_PAIRS choice
        :price:
            Decimal, float, integer or string representing number value.
        :order_type:
            exchanges.ORDER_TYPES choice
        :amount_in_contracts:
            (True|False) Whether the `amount`  argument is expressed in cryptos or contracts
        :closing:
            (True|False) Whether the order we are opening is to close an existing position or not
        """
        # validate arguments
        is_restricted_to_values(action, exchanges.ACTIONS)

        is_instance(amount, (Decimal, float, int, str))
        passes_test(amount, lambda x: Decimal(x))
        if amount_in_contracts:
            passes_test(amount, lambda x: Decimal(x) >= 1)

        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        is_instance(price, (Decimal, float, int, str))
        passes_test(price, lambda x: Decimal(x))

        is_restricted_to_values(order_type, exchanges.ORDER_TYPES)

        is_instance(amount_in_contracts, bool)
        is_instance(closing, bool)

        path = '/v1/future_trade.do'
        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        if closing:
            action = (self.ACTION['close_long']
                      if action == exchanges.SELL
                      else self.ACTION['close_short'])
        else:
            action = (self.ACTION['open_short']
                      if action == exchanges.SELL
                      else self.ACTION['open_long'])

        if not amount_in_contracts:
            amount = crypto_to_contracts(
                amount,
                getattr(self, '{}_ticker'.format(symbol_pair)).last,
                self.CONTRACT_UNIT_AMOUNTS[symbol_pair])

        match_price = 1 if order_type == 'market' else 0
        params = {
            'symbol': symbol_pair,
            'contract_type': 'quarter',
            'price': float(Decimal(price)),
            'match_price': match_price,  # if market, 'price' field is ignored
            'amount': int(amount),  # in contracts
            'type': action,
            'lever_rate': 10,  # default
        }
        params['api_key'] = self.api_key
        params['sign'] = self._sign_params(params)
        return self._post(path, params=params, model_class=OkexOrder)

    def cancel_order(self, order_id):
        is_instance(order_id, (str, ))
        passes_test(order_id, lambda x: int(x))

        # NOTE: OKEX doesn't provide a way of getting all orders from any
        #       symbol pair. We need to loop through all of them until we find it.
        order = None
        for symbol_pair in currencies.SYMBOL_PAIRS:
            orders = self.get_open_orders(symbol_pair=symbol_pair)
            orders = [order for order in orders if int(order.id) == int(order_id)]
            if orders:
                # found order in current symbol pair, no need to keep iterating
                order = orders[0]
                break

        if not order:
            raise ValueError('Could not find order with ID "{}"'.format(order_id))

        path = '/v1/future_cancel.do'
        params = {
            'symbol': order.symbol_pair,
            'contract_type': 'quarter',
            'order_id': int(order_id),
        }
        params['api_key'] = self.api_key
        params['sign'] = self._sign_params(params)
        return self._post(path, params=params)

    def cancel_all_orders(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        orders = self.get_open_orders(symbol_pair=symbol_pair)
        if not orders:
            return

        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        order_ids = ','.join([order.id for order in orders])
        path = '/v1/future_cancel.do'
        params = {
            'symbol': symbol_pair,
            'contract_type': 'quarter',
            'order_id': order_ids,
        }
        params['api_key'] = self.api_key
        params['sign'] = self._sign_params(params)
        return self._post(path, params=params)

    def get_open_positions(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        path = '/v1/future_position.do'
        params = {
            'symbol': symbol_pair,
            'contract_type': 'quarter',
        }
        params['api_key'] = self.api_key
        params['sign'] = self._sign_params(params)
        return self._post(path, params=params,
                          transformation=self._transform_open_positions,
                          model_class=OkexPosition)

    def close_position(self, position_id, symbol_pair):
        raise NotImplementedError('OKEX API does not support position IDs')

    def close_all_positions(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        positions = self.get_open_positions(symbol_pair=symbol_pair)
        for pos in positions:
            # as we want to close the position,
            # we need to performe the opposite action to the given one.
            action = exchanges.SELL if pos.action == exchanges.BUY else exchanges.BUY
            self.open_order(
                action, pos.amount, pos.symbol_pair, pos.price, exchanges.MARKET,
                amount_in_contracts=True, closing=True)


# OkexClient.btc_usd_ticker = cached_property_with_ttl(ttl=60)(lambda self: self.get_ticker(currencies.BTC_USD))
for symbol_pair in OkexClient.SYMBOLS_MAPPING:
    attr_name = '{}_ticker'.format(symbol_pair)
    fun = lambda self: self.get_ticker(symbol_pair)
    fun.__name__ = attr_name
    setattr(OkexClient, attr_name, cached_property_with_ttl(ttl=60)(fun))



from decimal import Decimal

from tests import BaseXchangeTestCase
from xchange.validators import *


class IsRestrictedToValuesTestCase(BaseXchangeTestCase):

    def test_is_restricted_to_values_matches(self):
        is_restricted_to_values(1, (1, 2, 3))
        is_restricted_to_values("Hello", ("Hello", "World"))

    def test_is_restricted_to_values_dont_match(self):
        with self.assertRaisesRegexp(ValueError, 'Invalid "Hello" value'):
            is_restricted_to_values("Hello", (1, 2, 3))
        with self.assertRaisesRegexp(ValueError, 'Invalid "1" value'):
            is_restricted_to_values(1, ("Hello", 2, 3, 4))

    def test_is_restricted_to_values_invalid_usage(self):
        with self.assertRaises(AssertionError):
            is_restricted_to_values("Hello", False)


class IsInstanceTestCase(BaseXchangeTestCase):

    def test_is_instance_matches(self):
        is_instance(True, (bool))
        is_instance(1, (int))
        is_instance("Hello", (str, int))
        is_instance("Hello", str)

    def test_is_instance_dont_match(self):
        with self.assertRaises(ValueError):
            is_instance(True, (float, str))
        with self.assertRaises(ValueError):
            is_instance("Hello", int)

    def test_is_instance_dont_invalid_usage(self):
        with self.assertRaises(AssertionError):
            is_instance(True, "Hello")
        with self.assertRaises(AssertionError):
            is_instance(True, (1, "Hello"))


class PassesTestTestCase(BaseXchangeTestCase):

    def test_passes_test_matches(self):
        passes_test(10, lambda x: x > 5)
        passes_test("Hello", lambda x: isinstance(x, str))
        passes_test("Hello", lambda x: len(x) == 5)
        passes_test("1.23", lambda x: Decimal(x))

    def test_passes_test_dont_match(self):
        with self.assertRaisesRegexp(ValueError, "Given value didn't pass provided test"):
            passes_test(10, lambda x: x > 15)
        with self.assertRaisesRegexp(ValueError, "Given value didn't pass provided test"):
            passes_test("Hello", lambda x: len(x) == 10)

    def test_passes_test_invalid_usage(self):
        with self.assertRaises(AssertionError):
            passes_test(1, "Hello")
        with self.assertRaises(AssertionError):
            passes_test(1, (1, 2, 3))
        with self.assertRaises(AssertionError):
            passes_test(1, None)



from decimal import Decimal

from xchange.models.base import (
    Ticker, AccountBalance, OrderBook, Order, Position)


class KrakenTicker(Ticker):
    """
    Original JSON response (transformed):
    {'a': ['3809.00000', '1', '1.000'],
     'b': ['3803.60000', '1', '1.000'],
     'c': ['3809.30000', '0.08000000'],
     'h': ['3830.30000', '3830.30000'],
     'l': ['3570.10000', '3525.10000'],
     'o': '3606.80000',
     'p': ['3717.00933', '3650.22371'],
     't': [6565, 14224],
     'v': ['2204.84583619', '4523.49526430']}
    """
    def normalize_response(self, json_response):
        return {
            'ask': json_response['a'][0],
            'bid': json_response['b'][0],
            'low': json_response['l'][0],
            'high': json_response['h'][0],
            'last': json_response['c'][0],
            'volume': json_response['v'][0],
        }


class KrakenOrderBook(OrderBook):
    """
    Original JSON response format:
    {
        "error": [],
        "result": {
            "XXBTZUSD": {
                "asks": [
                    ["4626.12300", "0.014", 1504020172],
                    ["4628.80800", "0.100", 1504020185],
                    ["4638.00000", "2.939", 1504020185],
                    ...
                ],
                "bids": [
                    ["4626.00600", "0.003", 1504020190],
                    ["4626.00000", "1.410", 1504020191],
                    ["4613.57900", "3.659", 1504020156],
                    ...
                ]
            }
        }
    }
    """

    def normalize_response(self, json_response):
        symbol = list(json_response['result'].keys())[0]
        return {
            'asks': [(Decimal(str(l[0])), Decimal(str(l[1])))
                     for l in json_response['result'][symbol]['asks']],
            'bids': [(Decimal(str(l[0])), Decimal(str(l[1])))
                     for l in json_response['result'][symbol]['bids']],
        }


class KrakenAccountBalance(AccountBalance):
    """
    Response items format:
    {
        'amount': '0.0',
        'currency': 'ZUSD',
    }
    """
    def normalize_response(self, json_response):
        return {
            'symbol': json_response['currency'],
            'amount': json_response['amount']
        }


class KrakenOrder(Order):
    """
    Response items format (transformed):
    {'id': 'OGYUJ3-LSWJV-4OD4DU',
     'status': 'open',
     'cost': '0.00000',
     'descr': {
       'leverage': 'none',
       'order': 'sell 0.00500000 XBTUSD @ limit 5000.0',
       'ordertype': 'limit',
       'pair': 'XBTUSD',
       'price': '5000.0',
       'price2': '0',
       'type': 'sell'},
     'expiretm': 0,
     'fee': '0.00000',
     'misc': '',
     'oflags': 'fciq',
     'opentm': 1505603660.2901,
     'price': '0.00000',
     'refid': None,
     'starttm': 0,
     'status': 'open',
     'userref': None,
     'vol': '0.00500000',
     'vol_exec': '0.00000000'}
    """
    def normalize_response(self, json_response):
        order = {
            'id': json_response['id']
        }
        keys = ['descr', 'vol', 'status']
        if all([k in json_response for k in keys]):
            order.update({
                'action': json_response['descr']['type'],
                'amount': json_response['vol'],
                'price': json_response['descr']['price'],
                'symbol_pair': json_response['descr']['pair'],
                'type': json_response['descr']['ordertype'],
                'status': json_response['status']
            })
        return order


class KrakenPosition(Position):
    """
    Response items format:
    {'cost': '62.82690',
     'fee': '0.16963',
     'id': 'T3Y3IJ-YAMOG-YJAOJN',
     'margin': '31.41345',
     'misc': '',
     'net': '-0.0153',
     'oflags': '',
     'ordertxid': 'OHUWBX-E54D5-J5VMWY',
     'ordertype': 'market',
     'pair': 'XXBTZUSD',
     'posstatus': 'open',
     'rollovertm': '1506307875',
     'terms': '0.0100% per 4 hours',
     'time': 1506293475.2531,
     'type': 'buy',
     'value': '62.8',
     'vol': '0.01700000',
     'vol_closed': '0.00000000'}
    """
    def normalize_response(self, json_response):
        return {
            'id': json_response['id'],
            'action': json_response['type'],
            'amount': json_response['vol'],
            'price': Decimal(str(json_response['cost'])) / Decimal(str(json_response['vol'])),
            'symbol_pair': json_response['pair'],
            'profit_loss': json_response['net'],
        }



import re
from decimal import Decimal

import responses

from tests import BaseXchangeTestCase
from tests.fixtures import bitfinex
from xchange.factories import ExchangeClientFactory
from xchange.constants import exchanges, currencies
from xchange.exceptions import BitfinexException, InvalidSymbolPairException
from xchange.models.bitfinex import (
    BitfinexTicker, BitfinexOrderBook, BitfinexAccountBalance, BitfinexOrder,
    BitfinexPosition)


class BaseBitfinexClientTestCase(BaseXchangeTestCase):
    def setUp(self):
        super(BaseBitfinexClientTestCase, self).setUp()
        self.ClientClass = ExchangeClientFactory.get_client(exchanges.BITFINEX)
        self.client = self.ClientClass('API_KEY', 'API_SECRET')

        for fixture in bitfinex.FIXTURE_RESPONSES:
            responses.add(
                method=fixture['method'],
                url=re.compile(fixture['url_regex']),
                json=fixture['json'],
                status=fixture['status'],
                content_type=fixture['content_type'])


class BitfinexClientTickerTestCase(BaseBitfinexClientTestCase):

    @responses.activate
    def test_get_ticker(self):
        ticker = self.client.get_ticker(currencies.BTC_USD)
        expected = {
            'ask': Decimal('9398.2'),
            'bid': Decimal('9398.1'),
            'high': Decimal('9500.0'),
            'last': Decimal('9398.1'),
            'low': Decimal('8750.0'),
            'volume': Decimal('44197.286828289994')
        }
        self.assertEqual(ticker, expected)
        self.assertEqual(type(ticker), BitfinexTicker)

    @responses.activate
    def test_get_ticker_invalid_symbol_pair(self):
        with self.assertRaisesRegexp(ValueError,
                                     'Invalid "usd_usd" value, expected any of'):
            self.client.get_ticker('usd_usd')


class BitfinexClientOrderBookTestCase(BaseBitfinexClientTestCase):

    @responses.activate
    def test_get_order_book(self):
        order_book = self.client.get_order_book(currencies.BTC_USD)
        expected = {
        'asks': [
            (Decimal('9329'), Decimal('7.50897281')),
            (Decimal('9328.7'), Decimal('0.08')),
            (Decimal('9327.4'), Decimal('0.09694343')),
            (Decimal('9327.3'), Decimal('0.31'))],
        'bids': [
            (Decimal('9327.1'), Decimal('0.15')),
            (Decimal('9326.9'), Decimal('0.10915328')),
            (Decimal('9326.8'), Decimal('0.34')),
            (Decimal('9326.2'), Decimal('0.57462657')),
            (Decimal('9326.1'), Decimal('0.01975221'))]
        }
        self.assertEqual(order_book, expected)
        self.assertEqual(type(order_book), BitfinexOrderBook)


class BitfinexClientAccountBalanceTestCase(BaseBitfinexClientTestCase):

    @responses.activate
    def test_get_account_balance(self):
        balance = self.client.get_account_balance()
        expected = [
            {'amount': Decimal('0.0'), 'symbol': 'bfx'},
            {'amount': Decimal('0.01209215'), 'symbol': 'btc'},
            {'amount': Decimal('0.00009361'), 'symbol': 'usd'}
        ]
        self.assertEqual(balance, expected)

        self.assertEqual(type(balance), list)
        for obj in balance:
            self.assertEqual(type(obj), BitfinexAccountBalance)

    @responses.activate
    def test_get_account_balance_symbol_pair(self):
        balance = self.client.get_account_balance(currencies.BTC)
        expected = {'amount': Decimal('0.01209215'), 'symbol': 'btc'}
        self.assertEqual(balance, expected)
        self.assertEqual(type(balance), BitfinexAccountBalance)


class BitfinexGetOpenOrdersTestCase(BaseBitfinexClientTestCase):

    @responses.activate
    def test_get_open_orders(self):
        fixture = [
            {
                'avg_execution_price': '0.0',
                'cid': 85481934272,
                'cid_date': '2017-09-16',
                'exchange': None,
                'executed_amount': '0.0',
                'gid': None,
                'id': 3864975544,
                'is_cancelled': False,
                'is_hidden': False,
                'is_live': True,
                'oco_order': None,
                'original_amount': '1.0',
                'price': '2.0',
                'remaining_amount': '1.0',
                'side': 'buy',
                'src': 'web',
                'symbol': 'btcusd',
                'timestamp': '1505625352.0',
                'type': 'limit',
                'was_forced': False
            }
        ]
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/orders'),
            json=fixture,
            status=200,
            content_type='application/json')
        open_orders = self.client.get_open_orders(currencies.BTC_USD)
        expected = [
            {
                'action': 'buy',
                'amount': Decimal('1.0'),
                'id': '3864975544',
                'price': Decimal('2.0'),
                'status': 'open',
                'symbol_pair': 'btc_usd',
                'type': 'limit'
            }
        ]
        self.assertEqual(type(open_orders), list)
        for obj in open_orders:
            self.assertEqual(type(obj), BitfinexOrder)
        self.assertEqual(open_orders, expected)

    @responses.activate
    def test_get_open_orders_filters_by_symbol_pair(self):
        fixture = [
            {
                'avg_execution_price': '0.0',
                'cid': 85481934272,
                'cid_date': '2017-09-16',
                'exchange': None,
                'executed_amount': '0.0',
                'gid': None,
                'id': 3864975544,
                'is_cancelled': False,
                'is_hidden': False,
                'is_live': True,
                'oco_order': None,
                'original_amount': '1.0',
                'price': '2.0',
                'remaining_amount': '1.0',
                'side': 'buy',
                'src': 'web',
                'symbol': 'btcusd',
                'timestamp': '1505625352.0',
                'type': 'limit',
                'was_forced': False
            },
            {
                'avg_execution_price': '0.0',
                'cid': 85478964272,
                'cid_date': '2017-09-16',
                'exchange': None,
                'executed_amount': '0.0',
                'gid': None,
                'id': 3864421544,
                'is_cancelled': False,
                'is_hidden': False,
                'is_live': True,
                'oco_order': None,
                'original_amount': '1.0',
                'price': '2.0',
                'remaining_amount': '1.0',
                'side': 'buy',
                'src': 'web',
                'symbol': 'ethusd',
                'timestamp': '1505625352.0',
                'type': 'limit',
                'was_forced': False
            },
        ]
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/orders'),
            json=fixture,
            status=200,
            content_type='application/json')
        open_orders = self.client.get_open_orders(currencies.ETH_USD)
        expected = [
            {
                'action': 'buy',
                'amount': Decimal('1.0'),
                'id': '3864421544',
                'price': Decimal('2.0'),
                'status': 'open',
                'symbol_pair': 'eth_usd',
                'type': 'limit'
            }
        ]
        self.assertEqual(type(open_orders), list)
        self.assertEqual(len(open_orders), 1)
        for obj in open_orders:
            self.assertEqual(type(obj), BitfinexOrder)
        self.assertEqual(open_orders, expected)

    @responses.activate
    def test_get_open_orders_empty_response(self):
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/orders'),
            json=[],  # no open orders
            status=200,
            content_type='application/json')
        open_orders = self.client.get_open_orders(currencies.BTC_USD)
        self.assertEqual(type(open_orders), list)
        self.assertEqual(open_orders, [])


class BitfinexOpenOrderTestCase(BaseBitfinexClientTestCase):

    @responses.activate
    def test_open_order(self):
        fixture = {
            'avg_execution_price': '0.0',
            'cid': 77112547681,
            'cid_date': '2017-08-20',
            'exchange': 'bitfinex',
            'executed_amount': '0.0',
            'gid': None,
            'id': 3438135637,
            'is_cancelled': False,
            'is_hidden': False,
            'is_live': True,
            'oco_order': None,
            'order_id': 3434679437,
            'original_amount': '0.01209215',
            'price': '4118.0',
            'remaining_amount': '0.01209215',
            'side': 'sell',
            'src': 'api',
            'symbol': 'btcusd',
            'timestamp': '1503225464.317116079',
            'type': 'limit',
            'was_forced': False
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/order/new'),
            json=fixture,
            status=200,
            content_type='application/json')
        order = self.client.open_order(
            action=exchanges.SELL,
            amount='0.01209215',
            symbol_pair=currencies.BTC_USD,
            price='4118.0',
            order_type=exchanges.LIMIT)
        expected = {
            'id': '3438135637',
            'action': 'sell',
            'amount': Decimal('0.01209215'),
            'price': Decimal('4118.0'),
            'status': 'open',
            'symbol_pair': 'btc_usd',
            'type': 'limit'
        }
        self.assertEqual(order, expected)
        self.assertEqual(type(order), BitfinexOrder)

    @responses.activate
    def test_open_order_bad_request(self):
        fixture = {'message': 'Key price should be a decimal string.'}
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/order/new'),
            json=fixture,
            status=400,
            content_type='application/json')
        with self.assertRaisesRegexp(BitfinexException, 'Got 400 response with content:'):
            self.client.open_order(
                action=exchanges.SELL,
                amount='0.01209215',
                symbol_pair=currencies.BTC_USD,
                price='0.000000',  # won't work
                order_type=exchanges.LIMIT)


class BitfinexCancelOrderTestCase(BaseBitfinexClientTestCase):

    @responses.activate
    def test_cancel_order(self):
        order_id = 3439467895
        fixture = {'result': 'Order {} submitted for cancellation; waiting for confirmation.'.format(order_id)}
        responses.add(
            method='POST',
            url=re.compile('https://api\.bitfinex\.com/v1/order/cancel'),
            json=fixture,
            status=200,
            content_type='application/json')
        response = self.client.cancel_order(order_id)
        expected = {
            'result': 'Order 3439467895 submitted for cancellation; waiting for confirmation.'}
        self.assertEqual(response, expected)

    @responses.activate
    def test_cancel_all_orders(self):
        fixture = [
            {
                'avg_execution_price': '0.0',
                'cid': 85481934272,
                'cid_date': '2017-09-16',
                'exchange': None,
                'executed_amount': '0.0',
                'gid': None,
                'id': 3864975544,
                'is_cancelled': False,
                'is_hidden': False,
                'is_live': True,
                'oco_order': None,
                'original_amount': '1.0',
                'price': '2.0',
                'remaining_amount': '1.0',
                'side': 'buy',
                'src': 'web',
                'symbol': 'btcusd',
                'timestamp': '1505625352.0',
                'type': 'limit',
                'was_forced': False
            },
            {
                'avg_execution_price': '0.0',
                'cid': 85478964272,
                'cid_date': '2017-09-16',
                'exchange': None,
                'executed_amount': '0.0',
                'gid': None,
                'id': 3864421544,
                'is_cancelled': False,
                'is_hidden': False,
                'is_live': True,
                'oco_order': None,
                'original_amount': '1.0',
                'price': '2.0',
                'remaining_amount': '1.0',
                'side': 'buy',
                'src': 'web',
                'symbol': 'ethusd',
                'timestamp': '1505625352.0',
                'type': 'limit',
                'was_forced': False
            },
        ]
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/orders'),
            json=fixture,
            status=200,
            content_type='application/json')
        fixture = {'result': 'Orders cancelled'}
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/order/cancel/multi'),
            json=fixture,
            status=200,
            content_type='application/json')
        response = self.client.cancel_all_orders(currencies.BTC_USD)
        self.assertEqual(response, fixture)


class BitfinexGetOpenPositionsTestCase(BaseBitfinexClientTestCase):

    @responses.activate
    def test_get_open_positions(self):
        fixture = [
            {
                'id': 34589787,
                'amount': '-0.01209215',
                'base': '4118.0',
                'pl': '-0.10928401484',
                'status': 'ACTIVE',
                'swap': '0.0',
                'symbol': 'btcusd',
                'timestamp': '1503264460.0'
            }
        ]
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/positions'),
            json=fixture,
            status=200,
            content_type='application/json')
        positions = self.client.get_open_positions(currencies.BTC_USD)
        expected = [
            {
                'id': '34589787',
                'action': 'sell',
                'amount': Decimal('0.01209215'),
                'price': Decimal('4118.0'),
                'profit_loss': Decimal('-0.10928401484'),
                'symbol_pair': 'btc_usd'
            }
        ]
        self.assertEqual(positions, expected)
        self.assertEqual(len(positions), 1)
        self.assertEqual(type(positions), list)
        for obj in positions:
            self.assertEqual(type(obj), BitfinexPosition)

    @responses.activate
    def test_get_open_positions_filters_symbol_pair(self):
        fixture = [
            {
                'id': 34589787,
                'amount': '-0.01209215',
                'base': '4118.0',
                'pl': '-0.10928401484',
                'status': 'ACTIVE',
                'swap': '0.0',
                'symbol': 'btcusd',
                'timestamp': '1503264460.0'
            },
            {
                'id': 34546527,
                'amount': '-0.01209215',
                'base': '4118.0',
                'pl': '-0.10928401484',
                'status': 'ACTIVE',
                'swap': '0.0',
                'symbol': 'ethusd',
                'timestamp': '1503264460.0'
            },
        ]
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/positions'),
            json=fixture,
            status=200,
            content_type='application/json')
        positions = self.client.get_open_positions(currencies.ETH_USD)
        expected = [
            {
                'id': '34546527',
                'action': 'sell',
                'amount': Decimal('0.01209215'),
                'price': Decimal('4118.0'),
                'profit_loss': Decimal('-0.10928401484'),
                'symbol_pair': 'eth_usd'
            }
        ]
        self.assertEqual(positions, expected)
        self.assertEqual(len(positions), 1)
        self.assertEqual(type(positions), list)
        for obj in positions:
            self.assertEqual(type(obj), BitfinexPosition)

    @responses.activate
    def test_get_open_positions_no_open_position(self):
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/positions'),
            json=[],
            status=200,
            content_type='application/json')
        positions = self.client.get_open_positions(currencies.BTC_USD)
        self.assertEqual(positions, [])
        self.assertEqual(len(positions), 0)
        self.assertEqual(type(positions), list)


class BitfinexClosePositionTestCase(BaseBitfinexClientTestCase):

    @responses.activate
    def test_close_position(self):
        fixture = {
            'avg_execution_price': '0.0',
            'cid': 81742456868,
            'cid_date': '2017-09-16',
            'exchange': 'bitfinex',
            'executed_amount': '0.0',
            'gid': None,
            'id': 3848456981,
            'is_cancelled': False,
            'is_hidden': False,
            'is_live': True,
            'oco_order': None,
            'order_id': 3841546911,
            'original_amount': '0.005',
            'price': '3710.5',
            'remaining_amount': '0.005',
            'side': 'buy',
            'src': 'api',
            'symbol': 'btcusd',
            'timestamp': '1505601744.272215012',
            'type': 'market',
            'was_forced': False
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/order/new'),
            json=fixture,
            status=200,
            content_type='application/json')
        fixture = [
            {
                'id': 34589787,
                'amount': '-0.01209215',
                'base': '4118.0',
                'pl': '-0.10928401484',
                'status': 'ACTIVE',
                'swap': '0.0',
                'symbol': 'btcusd',
                'timestamp': '1503264460.0'
            }
        ]
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/positions'),
            json=fixture,
            status=200,
            content_type='application/json')
        order = self.client.close_position(
            position_id='34589787',
            symbol_pair=currencies.BTC_USD)
        expected = {
            'action': 'buy',
            'amount': Decimal('0.005'),
            'id': '3848456981',
            'price': Decimal('3710.5'),
            'status': 'open',
            'symbol_pair': 'btc_usd',
            'type': 'market'
        }
        self.assertEqual(order, expected)
        self.assertEqual(type(order), BitfinexOrder)

    @responses.activate
    def test_close_all_positions(self):
        fixture = [
            {
                'id': 34589787,
                'amount': '-0.01209215',
                'base': '4118.0',
                'pl': '-0.10928401484',
                'status': 'ACTIVE',
                'swap': '0.0',
                'symbol': 'btcusd',
                'timestamp': '1503264460.0'
            }
        ]
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/positions'),
            json=fixture,
            status=200,
            content_type='application/json')
        fixture = {
            'avg_execution_price': '0.0',
            'cid': 81742456868,
            'cid_date': '2017-09-16',
            'exchange': 'bitfinex',
            'executed_amount': '0.0',
            'gid': None,
            'id': 3848456981,
            'is_cancelled': False,
            'is_hidden': False,
            'is_live': True,
            'oco_order': None,
            'order_id': 3841546911,
            'original_amount': '0.005',
            'price': '3710.5',
            'remaining_amount': '0.005',
            'side': 'buy',
            'src': 'api',
            'symbol': 'btcusd',
            'timestamp': '1505601744.272215012',
            'type': 'market',
            'was_forced': False
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.bitfinex.com/v1/order/new'),
            json=fixture,
            status=200,
            content_type='application/json')
        self.client.close_all_positions(currencies.BTC_USD)



import setuptools
import xchange


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name='xchange',
    version=xchange.__version__,
    description=("Many cryptocurrency exchange APIs, a single and unified API client"),
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/martinzugnoni/xchange',
    download_url=(
        "https://github.com/martinzugnoni/xchange/tarball/{version}".format(
            version=xchange.__version__)),
    author='Martin Zugnoni',
    author_email='martin.zugnoni@gmail.com',
    license='MIT',
    packages=setuptools.find_packages(),
    maintainer='Martin Zugnoni',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



from decimal import Decimal

from xchange.models.base import (Ticker, AccountBalance, OrderBook, Order,
                                 Position)


class BitfinexTicker(Ticker):
    """
    Original JSON response:
    {'ask': '3780.2',
     'bid': '3780.1',
     'low': '3490.0',
     'high': '3808.7',
     'last_price': '3780.2',
     'volume': '41981.56526245',
     'mid': '3780.15',
     'timestamp': '1506170236.287525866'}
    """
    def normalize_response(self, json_response):
        return {
            'ask': json_response['ask'],
            'bid': json_response['bid'],
            'low': json_response['low'],
            'high': json_response['high'],
            'last': json_response['last_price'],
            'volume': json_response['volume'],
        }


class BitfinexAccountBalance(AccountBalance):
    """
    Response items format:
    {
        'amount': '0.0',
        'available': '0.0',
        'currency': 'btc',
        'type': 'deposit'
    },
    """
    def normalize_response(self, json_response):
        return {
            'symbol': json_response['currency'],
            'amount': json_response['available']
        }


class BitfinexOrderBook(OrderBook):
    """
    Original JSON response format:
    {
        "asks": [
            {"price": "4610.6", "amount": "14.96", "timestamp": "1504020265.0"},
            {"price": "4610.7", "amount": "2.27682022", "timestamp": "1504020081.0"},
            {"price": "4611.0", "amount": "5.0", "timestamp": "1504020166.0"},
            ...
        ],
        "bids": [
            {"price": "4610.4", "amount": "1.538", "timestamp": "1504020260.0"},
            {"price": "4610.2", "amount": "0.37226385", "timestamp": "1504020265.0"},
            {"price": "4608.1", "amount": "0.094174", "timestamp": "1504020261.0"},
            ...
        ]
    }
    """

    def normalize_response(self, json_response):
        return {
            'asks': [(Decimal(doc['price']), Decimal(doc['amount']))
                     for doc in json_response['asks']],
            'bids': [(Decimal(doc['price']), Decimal(doc['amount']))
                     for doc in json_response['bids']],
        }


class BitfinexOrder(Order):
    """
    Response items format:
    {'avg_execution_price': '0.0',
      'cid': 81351934272,
      'cid_date': '2017-09-16',
      'exchange': None,
      'executed_amount': '0.0',
      'gid': None,
      'id': 3848275544,
      'is_cancelled': False,
      'is_hidden': False,
      'is_live': True,
      'oco_order': None,
      'original_amount': '1.0',
      'price': '2.0',
      'remaining_amount': '1.0',
      'side': 'buy',
      'src': 'web',
      'symbol': 'btcusd',
      'timestamp': '1505601352.0',
      'type': 'limit',
      'was_forced': False}
    """
    def normalize_response(self, json_response):
        return {
            'id': json_response['id'],
            'action': json_response['side'],
            'amount': json_response['original_amount'],
            'price': json_response['price'],
            'symbol_pair': json_response['symbol'],
            'type': json_response['type'],
            'status': 'open' if json_response['is_live'] else 'closed'
        }


class BitfinexPosition(Position):
    """
    Response items format:
    {'amount': u'-0.01209215',  # (-) sell (+) buy
     'base': u'4118.0',  # real "price" the position was opened
     'id': 36860087,
     'pl': u'-0.10928401484',
     'status': u'ACTIVE',
     'swap': u'0.0',
     'symbol': u'btcusd',
     'timestamp': u'1503264460.0'}]
    """
    def normalize_response(self, json_response):
        if Decimal(json_response['amount']) > 0:
            action = 'buy'
        else:
            action = 'sell'
        return {
            'id': json_response['id'],
            'action': action,
            'amount': abs(Decimal(json_response['amount'])),
            'price': json_response['base'],
            'symbol_pair': json_response['symbol'],
            'profit_loss': json_response['pl'],
        }
